package org.astakhova.data;

import java.util.List;

/**
 * Makes a description of a diagram
 * @author Astakhova Julia A.
 */
public interface IDiagram {
	
	/**
	 * Gets a node from the diagram with given coordinates
	 * @param x X-coordinate
	 * @param y Y-coordinate
	 * @return found node or null-value if it doesn't exist
	 */
	public INode getNode(int x, int y);
	
	/**
	 * Gets all nodes from the diagram packed in list
	 * @return list of nodes
	 */
	public List<INode> getNodes();
	
	/**
	 * Adds a new datanode to the diagram
	 * @param x X-coordinate
	 * @param y Y-coordinate
	 * @return created data node
	 */
	public IDataNode createDataNode(int x, int y);
	
	/**
	 * Adds a new condition node to the diagram
	 * @param x X-coordinate
	 * @param y Y-coordinate
	 * @return created condition node
	 */
	public IConditionNode createCondNode(int x, int y);
	
	/**
	 * Adds a new beginning node to the diagram
	 * @param x X-coordinate
	 * @param y Y-coordinate
	 * @return created beginning node
	 */
	public IBeginNode createBeginNode(int x, int y);
	
	/**
	 * Adds a new ending node to the diagram
	 * @param x X-coordinate
	 * @param y Y-coordinate
	 * @return created ending node
	 */
	public IEndNode createEndNode(int x, int y);
	
	/**
	 * Creates a new arrow
	 * @param source node the arrow comes from
	 * @param dest node the arrow gets to
	 * @return created arrow
	 */
	public IArrow createArrow(INode source, INode dest);
	
	/**
	 * Creates a new arrow coming from condition node. 'Cause label for the arrow isn't
	 *  set it will be of default branch value ("Yes" or "No")
	 * @param source condition node the arrow comes from
	 * @param dest node the arrow gets to
	 * @param branch condition branch to be chosen
	 * @return created arrow
	 */
	public IArrow createArrow(IConditionNode source, 
			INode dest, boolean branch);
	
	/**
	 * Creates a new arrow coming from condition node
	 * @param source condition node the arrow comes from
	 * @param dest node the arrow gets to
	 * @param branch condition branch to be chosen
	 * @param label a label that is bound to the arrow
	 * @return created arrow
	 */
	public IArrow createArrow(IConditionNode source, 
			INode dest, boolean branch, String label);
	
	/**
	 * Removes a node from the diagram.
	 * Also removes all arrows connected with the removed node
	 * @param n a node to remove 
	 */
	public void removeNode(INode n);
	
	/**
	 * Removes an arrow from the diagram
	 * @param a an arrow to remove
	 */
	public void removeArrow(IArrow a);
	
}
