package org.astakhova.data.impl;

import java.util.*;
import org.astakhova.data.IArrow;
import org.astakhova.data.INode;


/**
 * Makes a description of a node of a block diagram
 * @author Astakhova Julia A.
 */
public abstract class Node implements INode{
	private String myText = new String();
	private int myX;
	private int myY;
	private int myHeight;
	private int myWidth;
	
	/**
	 *A list of input arrows 
	 */
	private LinkedList<IArrow> inArrow = new LinkedList<IArrow>();
	
	/**
	 *A list of output arrows 
	 */
	private LinkedList<IArrow> outArrow = new LinkedList<IArrow>();
	
	/**
	 * Creates a new node with fixed coordinates
	 * @param x X-coordinate
	 * @param y Y-coordinate
	 */
	Node(int x, int y) {
		myX = x;
		myY = y;
	}
	
	/**
	 * Gets a text value
	 * @return a string with the text value
	 */
	public String getText() {
		return myText;
	}
	
	/**
	 * Changes a text value
	 * @param text a new text value
	 */
	public void setText(String text) {
		myText = text;
	}
	
	/**
	 * Gets an X-coordinate of the node
	 * @return X-coordinate
	 */
	public int getX() {
		return myX;
	}
	
	/**
	 * Gets an Y-coordinate of the node
	 * @return Y-coordinate
	 */
	public int getY() {
		return myY;
	}
	
	/**
	 * Sets a new values for the coordinate X. 
	 * @param x X-coordinate
	 */
	public void setX(int x) {
			myX = x;
	}
	
	/**
	 * Sets a new values for the coordinate Y. 
	 * @param y Y-coordinate
	 */
	public void setY(int y) {
			myY = y;
	}
	
	/**
	 * Connects the node with another one by arrow
	 * @param node a destination-node for the arrow
	 * @return created arrow
	 */
	Arrow connect(Node node) {
		if (canBind() && node.canBeBound()) {
			Arrow a = new Arrow(this, node);
			outArrow.add(a);
			node.inArrow.addLast(a);
			return a;
		}
		return null;
	}
	
	/**
	 * Checks if there can be created new output arrows
	 * @return true if everything's OK
	 */
	public boolean canBind() {
		return outArrow.isEmpty();
	}
	
	/**
	 * Checks if there can be created new input arrows
	 * @return true if it's so
	 */
	public boolean canBeBound() {
		return true;
	}
	
	/**
	 * Removes all arrows connecting this node with others
	 */
	void unBind() {
		for (IArrow a : inArrow) {
			((Node) a.getSource()).removeOutArrow(a);
		}
		for (IArrow b : outArrow)
			if (b!= null) {
				((Node) b.getDest()).removeInArrow(b);
			}
	}
	
	public String toString() {
		String s = "The node description:\n";
		s += "its text: " + myText + "\n";
		s += "its coordinates: " + myX + " " + myY +"\n";
		for (IArrow a : outArrow)
			s += "the outArrow:\n" + a + "\n";
		return s;
	}
	
	void addInArrow(Arrow a) {
		inArrow.add(a);
	}
	
	void addOutArrow(Arrow a) {
		outArrow.add(a);
	}
	
	int indexOfOutArrow(Arrow a) {
		return outArrow.indexOf(a);
	}
	
	void setOutArrow(int n, Arrow a) {
		outArrow.set(n, a);
	}
	
	IArrow getOutArrow(int n) {
		return outArrow.get(n);
	}
	
	void removeInArrow(IArrow a) {
		inArrow.remove(a);
	}
	
	void removeOutArrow(IArrow a) {
		if (this instanceof ConditionNode)
			setOutArrow(indexOfOutArrow((Arrow) a), null);
		else
			outArrow.remove(a);
	}
	
	public int getHeight() {
		return myHeight;
	}
	
	public int getWidth() {
		return myWidth;
	}
	
	public void setHeight(int height) {
		myHeight = height;
	}
	
	public void setWidth(int width) {
		myWidth = width;
	}
	
	public List<IArrow> getOutArrows() {
		return Collections.unmodifiableList(outArrow);
	}
}
